/* Soubor je ulozen v kodovani UTF-8.
 * Kontrola kódování: Příliš žluťoučký kůň úpěl ďábelské ódy.
 */
package cz.pecinovsky.english.pojj.adv_framework.test_util;

import cz.pecinovsky.english.pojj.adv_framework.utilities.DBG_Logger;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import static cz.pecinovsky.english.pojj.adv_framework.utilities.FormatStrings.*;



/*******************************************************************************
 * Třída {@code ATest} slouží jako rodičovská třída
 * různých testovacích tříd kterým poskytuje společné metody,
 * tj. metody pro zadání reakce na chybu a
 * metodu pro nastavení příznaku práce v grafickém režimu.
 *
 * @author    Rudolf PECINOVSKÝ
 * @version   0.00.000
 */
class ATest
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================

    /** Logger, prostřednictvím nějž zanamenáváme veškeré informace. */
    protected final static DBG_Logger DBG = DBG_Logger.getInstance();



//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================

    /** Příznak provozu v grafickém režimu. */
    private static boolean GUI = false;



//== STATICKÝ INICIALIZAČNÍ BLOK - STATICKÝ KONSTRUKTOR ========================

    /***************************************************************************
     * Statický konstruktor nastavuje:
     *  - implicitní ošetření nezachycených přerušení
     */
    static {
        Thread.setDefaultUncaughtExceptionHandler(
            new Thread.UncaughtExceptionHandler() {
                @Override
                public void uncaughtException(Thread t, Throwable e)
                {
//                    e.printStackTrace(System.out);
//                    DBG.severe("\n==================================" +
//                            "\nVe vlákně {0} byla vyhozena výjimka:\n{1}\n{2}",
//                            t, e, DBG.stackTrace(e));
                    DBG.severe(DBG.stackTrace(e));
                }
            }
        );
    }



//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

    /** Seznam chyb nalezených v průběhu testování danou instancí testu. */
    protected final List<String> foundErrors = new ArrayList<String>();



//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ TŘÍDY ========================================

    /***************************************************************************
     * Nastaví příznak provozu v grafickém režimu.
     *
     * @param GUI Nastavovaná hodnota příznaku
     */
    public static void setGUI(boolean GUI)
    {
        ATest.GUI = GUI;
    }



//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    /***************************************************************************
     * Rodičovský konstruktor nemusí provádět žádné speciální akce.
     */
    protected ATest()
    {
    }



//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ INSTANCÍ =====================================
//== OSTATNÍ NESOUKROMÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Vytiskne chybové hlášení a v případě, že je program provozován pod
     * grafickým uživatelskám rozhraním, vypíše chybovou zprávu v okně.
     *
     * @param format     Formát tisku chybového hlášení. Bude ještě doplněn
     *                   společnou úvodní a závěrečnou sekvencí.
     * @param parameters Případné další parametry k tisku
     */
    protected void ERROR(String format, Object... parameters)
    {
        error(format, parameters);
    }


    /***************************************************************************
     * Vytiskne chybové hlášení a vyhodí výjimku {@code TestException}.
     *
     * @param format    Formát tisku chybového hlášení. Bude ještě doplněn
     *                  společnou úvodní a závěrečnou sekvencí.
     * @param parametry Případné další parametry k tisku
     * @throws TestException
     */
    protected void ERROR_T(String format, Object... parametry)
    {
        String message = error(format, parametry);
        throw new TestException(message);
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================

    /***************************************************************************
     * Vytiskne a vrátí chybové hlášení a v případě, že je program provozován
     * pod grafickým uživatelskám rozhraním, vypíše chybovou zprávu v okně.
     *
     * @param format     Formát tisku chybového hlášení. Bude ještě doplněn
     *                   společnou úvodní a závěrečnou sekvencí.
     * @param parameters Případné další parametry k tisku
     * @return Chybové hlášení
     */
    private String error(String format, Object[] parameters)
    {
        String message = String.format(format, parameters);
        foundErrors.add(message);
        String complet = N_BERFORE_N + message + N_AFTER_N;
        DBG.severe(complet);
        if (GUI) {
            JOptionPane.showMessageDialog(null, message);
        }
        return message;
    }


//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================
//== INTERNÍ DATOVÉ TYPY =======================================================
//== TESTOVACÍ METODY A TŘÍDY ==================================================
//
//    /***************************************************************************
//     * Testovací metoda.
//     */
//    public static void test()
//    {
//        ATest inst = new ATest();
//    }
//    /** @param args Parametry příkazového řádku - nepoužívané. */
//    public static void main( String[] args )  {  test();  }
}
