/* Soubor je ulozen v kodovani UTF-8.
 * Kontrola kódování: Příliš žluťoučký kůň úpěl ďábelské ódy.
 */
package cz.pecinovsky.english.pojj.adv_framework.test_util;

import cz.pecinovsky.english.pojj.adv_framework.game_txt.IGame;

import cz.pecinovsky.english.pojj.adv_framework.scenario.AScenarioManager;
import cz.pecinovsky.english.pojj.adv_framework.scenario.ScenarioStep;
import cz.pecinovsky.english.pojj.adv_framework.scenario.Scenario;
import cz.pecinovsky.english.pojj.adv_framework.scenario.TypeOfStep;
import cz.pecinovsky.english.pojj.adv_framework.scenario.TypeOfScenario;
import cz.pecinovsky.english.pojj.adv_framework.test_util.ScenarioTest.Summary;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import java.util.Date;
import java.util.List;

import static cz.pecinovsky.english.pojj.adv_framework.scenario.TypeOfStep.*;

import static cz.pecinovsky.english.pojj.adv_framework.utilities.FormatStrings.*;


/*******************************************************************************
 * Instance třídy {@code GameTRunTest} slouží k testování
 * her implementujících rozhraní {@code IGame} a ovladatelných podle scénářů
 * spravovaných instancí potomka třídy {@link AScenarioManager}.
 * Instance jsouschopny prověřit, zda při spuštění libovolného testovacího
 * scénáře hra reaguje tak, jak je ve scénáři naplánováno.  Za testovací scénář
 * se přitom považuje libovolný scénář, který není demonstrační.
 *
 * @author    Rudolf PECINOVSKÝ
 * @version   5.0
 */
class GameTRunTest extends ATest
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================

    /** Příznak práce v textovém (tj. ne okenním) režimu. */
    private static final boolean TEXT_IU = true;



//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================
//== STATICKÝ INICIALIZAČNÍ BLOK - STATICKÝ KONSTRUKTOR ========================
//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

    /** Odkaz na instanci testované hry. */
    private final IGame game;

    /** Úplný název třídy testované hry. */
    private final String gamename;

    /** Správce scénářů testované hry. */
    private final AScenarioManager manager;

    /** Autor testované hry. */
    private String author;

    /** X-name autora testované hry. */
    private String xname;



//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================

    /** Proměnná udržující informaci o tom, je-li hra ve stavu před startem,
     *  tj. ještě nezačala, anebo již skončila a může znovu začít. */
    private boolean notRunning;



//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ TŘÍDY ========================================
//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    /***************************************************************************
     * Vytvoří instanci, která bude testovat zadanou hru.
     *
     * @param hra Hra, která se bude testovat
     */
    public GameTRunTest (IGame hra)
    {
        this.game     = hra;
        this.gamename = hra.getClass().getName();
        this.manager  = hra.getScenarioManager();
        this.author   = hra.getAuthor();
        this.xname    = hra.getXNAME();

        if (!(author.equals(manager.getAuthor())  &&
              xname.equals(manager.getXNAME())) )
        {
            throw new RuntimeException(
                    "\nAutor hry není totožný s autorem správce scénářů");
        }
    }


//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ INSTANCÍ =====================================
//== OSTATNÍ NESOUKROMÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Souhrn akcí, které je potřeba provést před vlastním odstartováním hry.
     * V této třídě nemusí metoda dělat nic, ale potomci mohou mít požadavky.
     */
    protected void beforeStart()
    {
        GameStepTest.reset();
    }


    /***************************************************************************
     * Prověří správnou funkci hry v zadanémé kroku testu.
     *
     * @param ks Prováděný krok scénáře
     * @throws IllegalStateException v případe, kdy stav hry neodpovídá
     *         stavu očekávanému podle hodnot dané instance kroku testu
     */
    protected void verifyScenarioStep(ScenarioStep ks)
    {
        String message    = game.executeCommand(ks.getCommand());
        String inspection = GameStepTest.verify(ks, game, message);
        DBG.info(inspection);
        if (game.isReady()) {
            notRunning = true;
        }
    }


    /***************************************************************************
     * Souhrn akcí, které je potřeba provést po skončení hry.
     * V této třídě nemusí metoda dělat nic, ale potomci mohou mít požadavky.
     */
    protected void afterEnd()
    {
    }


    /***************************************************************************
     * Souhrn akcí, které je potřeba provést po tisku zprávy o ukončení hry;
     * metoda vrací informaci o tom, byla-li v průběhu testu zjištěna
     * najaká méně závažná chyba (závažné chyby končí vyhozením výjimky).
     * V této třídě nemusí metoda dělat nic, ale potomci mohou mít požadavky.
     *
     * @return Byly-li zjištěny nějaké chyby, vrátí text s jejich popisy,
     *         v opačném případě vrátí prázdný řetězec
     */
    public String finalSummary()
    {
        String text;
        if (foundErrors.isEmpty()) {
            text = "";
        }
        else {
            StringBuilder sb = new StringBuilder("\nNalezené chyby:" +
                                                 N_LINE_N);
            for (String s : foundErrors) {
                sb.append(s).append(N_LINE_N);
            }
            text = sb.toString();
        }
        return text;
    }


    /***************************************************************************
     * Spustí komunikaci mezi implicitní hrou a
     * hypotetickým uživatelem reprezentovaným instancí tohoto testu
     * řízenou nultým scénářem.
     *
     * @return Informace, zda test proběhl bez chyb
     */
    public boolean executeBasicScenario()
    {
        return executeScenario(0);
    }


    /***************************************************************************
     * Spustí hru řízenou jejím scénářem se zadaným indexem;
     * této hře bude postupně zadavat příkazy scénáře
     * a kontrolovat její odpovědi.
     *
     * @param index Index scénáře, kterým testujeme hru
     * @return Informace, zda test proběhl bez chyb
     */
    public boolean executeScenario(int index)
    {
        Scenario scenario = manager.getScenario(index);
        return executeScenario(scenario);
    }


    /***************************************************************************
     * Spustí hru řízenou zadaným scénářem; této hře bude postupně
     * zadavat příkazy scénáře a porovnávat její odpovědi s pořadavky scénáře.
     *
     * @param scenario Scénář, kterým testujeme hru
     * @return Informace, zda test proběhl bez chyb
     */
    public boolean executeScenario(Scenario scenario)
    {
        boolean errorOccured;
        Exception exception = null;
        foundErrors.clear();

        String text = header(scenario) +
                      "===== Test prováděných operací =====";
        DBG.info(text);

        try {
            beforeStart();

            notRunning = true;
            int index = 1;
            for (ScenarioStep ks : scenario) {
                ks.setNewIndex(index++);
                notRunning = verifyIsReady(notRunning);
                verifyScenarioStep(ks);        //Tiskne zprávu
            }
            if (! game.isReady()) {
                ERROR("Po ukončení scénáře hra tvrdí, že stále běží." +
                    "\nScénář musí odevzdat ukončenou hru.");
            }

            afterEnd();
        } catch( Exception e) {
            exception = e;
        } finally {
            errorOccured = ! foundErrors.isEmpty();
            DBG.info("\n===== Konec testu prováděných operací ====={0}",
                     header(scenario) );
            if (exception != null) {
                errorOccured = true;
                StringWriter sw = new StringWriter();
                PrintWriter  pw = new PrintWriter(sw);
                exception.printStackTrace(pw);
                DBG.severe("Ukončení bylo způsobeno vyhozením výjimky:\n{0}",
                           sw.toString());
                pw.close();
            }
            String chyby = finalSummary();
            errorOccured |= ! "".equals(chyby);
            String txt = "\nPodle výsledků testu " +
                    (errorOccured
                    ?  "PROGRAM OBSAHUJE JISTÉ CHYBY\n" + chyby
                    :  "program pravděpodobně neobsahuje žádné závažné chyby") +
                    N_HASHES_N + HASHES_N;
//                "\n===========================================" +
//                  "===========================================\n";
            DBG.info(txt);
        }
        if (errorOccured) {
            game.stop();
        }
        return errorOccured;
    }


    /***************************************************************************
     * Provede všechny scénáře spravované správcem scénářů testované hry.
     *
     * @return Informace, zda testy proběhly bez chyb
     */
    public boolean  executeAllScenarios()
    {
        boolean errorOccured = false;
        for (Scenario scenario : manager) {
            errorOccured |= executeScenario(scenario);
        }
        return errorOccured;
    }


    /***************************************************************************
     * Spustí hru podle scénaře základního úspěšného, základního chybového
     * a vlastního rozšířeného.
     *
     * @return Informace, zda testy proběhly bez chyb
     */
    public boolean executeExtended()
    {
        //Otestuje základní úspěšný a základní chybový scénář
        ScenarioTest.Summary summary = ScenarioTest.testBasicScenarios(manager);

        //Vytvoří doplňující kontrolní scénář
        Scenario kontrolní = scenarioWithAccessoryTests(summary);

        boolean errorOccured = false;
        errorOccured |= executeScenario(0);
        errorOccured |= executeScenario(1);
        errorOccured |= executeScenario(kontrolní);

        //Vytvoří scénář pro vyřešení zadání obhajoby
//        Scenario obhajoba  =
////                scénářObhajoby_4(souhrn);
//                scénářObhajoby_6(souhrn);
//        sChybou |= executeScenario(obhajoba);

        String zpráva = N_HASHES_N +
            "Při testu programu prostřednictvím na počátku uvedených scénářů\n"+
            (errorOccured ? "BYLY NALEZENY JISTÉ CHYBY\n" +
                            "Zjištěne chyby jsou uvedeny vždy ve výpisu\n" +
                            "na konci zprávy o průběhu hry podle daného scénáře"
                          : "nebyla nalezena žádná chyba.") +
            N_HASHES_N;
        DBG.info(zpráva);
        return errorOccured;
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================
//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Vrátí hlavičku testu zadaného scénáře.
     *
     * @param scenario
     * @return String s hlavičkou testu zadaného scénáře
     */
    private String header(Scenario scenario)
    {
        String text = N_HASHES_N +
                 "##### Čas:    " + new Date() +
               "\n##### Hra:    " + gamename   +
               "\n##### Autor:  " + xname      + " - " + author +
               "\n##### Scénář: " + scenario.getName() +
               N_HASHES_N;
        return text;
    }


    /***************************************************************************
     * Prověří, jestli testovaná hra nehlásí před startem svoji spuštěnost
     * nebo před koncem svoji ukončenost.
     *
     * @param start {@code true} =Test spuštěnosti před zadáním prvního příkazu,
     *              {@code false}=Test ukončenosti v dalším průběhu hry
     * @return
     */
    @SuppressWarnings("AssignmentToMethodParameter")
    private boolean verifyIsReady(boolean start)
    {
        if (start) {
            if (!game.isReady()) {
                ERROR_T("Hra se před spuštěním tváří jako spuštěná");
            }
            else {
                start = false;
            }
        }
        else if (game.isReady()) {
            ERROR("Předčasný konec hry");
        }
        return start;
    }


    /***************************************************************************
     * Přidá na konec dodaného seznamu kroků scénáře (předpokládáme prázdného)
     * testovací kroky, které odstarují hru, ukončí ji a opět odstartují.
     * Tím prověří reakci hry na pokus o její restart.
     *
     * @param summary Souhrn infomrací o hře získaný z dříve testovaných scénářů
     * @param initialSpace  Prostor, v němž hra začíná
     * @param initialStep   Nultý, tj. startovní (bezejmenný) krok scénáře
     * @param steps         Seznam kroků budoucího scénáře, na jehož konec
     *                      metoda přidá kroky vytvořené v této metodě
     * @return Vrátí krok scénáře, který explicitně ukončí hru,
     *         aby mohl být ještě jednou zařazen na konec scénáře.
     */
    private ScenarioStep addRestartOfGame(Summary summary, String initialSpace,
                            ScenarioStep initialStep, List<ScenarioStep> steps)
    {
        //Test restartu hry
        ScenarioStep finalStep = new ScenarioStep(99,
                     summary.basicCommands.get(TypeOfStep.tsEND),
                     ScenarioStep.IGNORED_MESSAGE,
                     initialSpace,
                     initialStep.neighbors,
                     initialStep.objects,
                     initialStep.bag,
                     TypeOfStep.tsEND);
        //Test restartovatelnosti hry
        steps.add(initialStep);
        steps.add(finalStep);
        steps.add(initialStep);
        return finalStep;
    }


    /***************************************************************************
     * Přidá no konec dodaného seznamu kroků scénáře testovací krok,
     * v němž prověří reakci hry na pokus o zadání základních příkazů
     * pro přechod do jiného prostoru a pro zvednutí a položení objektu,
     * kterým bude chybět příslušný parametr.
     *
     * @param summary Souhrn infomrací o hře získaný z dříve testovaných scénářů
     * @param initialSpace  Prostor, v němž hra začíná
     * @param initialStep   Nultý, tj. startovní (bezejmenný) krok scénáře
     * @param steps         Seznam kroků budoucího scénáře, na jehož konec
     *                      metoda přidá kroky vytvořené v této metodě
     */
    private void addArgumentLessStatement(Summary summary, String initialSpace,
                            ScenarioStep initialStep, List<ScenarioStep> steps)
    {
        TypeOfStep[] correctTypes = {tsMOVE,    tsPUT_DOWN,    tsPICK_UP};
        TypeOfStep[] enteredTypes = {tsMOVE_WA, tsPUT_DOWN_WA, tsPICK_UP_WA};
        //Kroky s bezparametrickými příkazy
        for (int i=0;   i < correctTypes.length;   i++) {
            String     command    =summary.basicCommands.get(correctTypes[i]);
            TypeOfStep typeOfStep = enteredTypes[i];

            ScenarioStep step = new ScenarioStep(99,
                                                 command,
                                                 ScenarioStep.IGNORED_MESSAGE,
                                                 initialSpace,
                                                 initialStep.neighbors,
                                                 initialStep.objects,
                                                 initialStep.bag,
                                                 typeOfStep);
            steps.add(step);
        }
    }


    /***************************************************************************
     * Přidá na konec dodaného seznamu kroků scénáře testovací krok,
     * v němž prověří reakci hry na pokus o přechod do aktuální prostoru.
     *
     * @param summary Souhrn infomrací o hře získaný z dříve testovaných scénářů
     * @param initialSpace  Prostor, v němž hra začíná
     * @param initialStep   Nultý, tj. startovní (bezejmenný) krok scénáře
     * @param steps         Seznam kroků budoucího scénáře, na jehož konec
     *                      metoda přidá kroky vytvořené v této metodě
     */
    private void addEnteringActualSpace(Summary summary, String initialSpace,
                            ScenarioStep initialStep, List<ScenarioStep> steps)
    {
        //Přechod do aktuální místnosti
        String přejdiDo = summary.basicCommands.get(TypeOfStep.tsMOVE) + " ";
        ScenarioStep krok = new ScenarioStep(99,
               přejdiDo + initialSpace,
               ScenarioStep.IGNORED_MESSAGE,
               initialSpace,
               initialStep.neighbors,
               initialStep.objects,
               initialStep.bag,
               TypeOfStep.tsBAD_NEIGHBOR);
        steps.add(krok);
    }


    /***************************************************************************
     * Přidá na konec dodaného seznamu kroků scénáře testovací krok,
     * v němž prověří reakci hry na pokus o přechod do prostoru,
     * který není sousednímprostorem aktuálního prostoru.
     *
     * @param summary Souhrn infomrací o hře získaný z dříve testovaných scénářů
     * @param initialSpace  Prostor, v němž hra začíná
     * @param initialStep   Nultý, tj. startovní (bezejmenný) krok scénáře
     * @param steps         Seznam kroků budoucího scénáře, na jehož konec
     *                      metoda přidá kroky vytvořené v této metodě
     */
    private void addEnteringAFarSpace(Summary summary, String initialSpace,
                            ScenarioStep initialStep, List<ScenarioStep> steps)
    {
        //Přechod do nesousedního prostoru
        for (int i=0;   i < initialStep.neighbors.length;   i++) {
            initialStep.neighbors[i] = initialStep.neighbors[i].toLowerCase();
        }
        String destination = null;
        Collection<String> neighbors = Arrays.asList(initialStep.neighbors);
        for (String place : summary.mentionedPlaces) {
            if (! place.equalsIgnoreCase(initialStep.place)  &&
                ! neighbors.contains(place.toLowerCase()))
            {
                destination = place;
            }
        }
        if (destination != null) {
            String goInto = summary.basicCommands.get(TypeOfStep.tsMOVE) + " ";
            ScenarioStep step = new ScenarioStep(99,
                   goInto + destination,
                   ScenarioStep.IGNORED_MESSAGE,
                   initialSpace,
                   initialStep.neighbors,
                   initialStep.objects,
                   initialStep.bag,
                   TypeOfStep.tsBAD_NEIGHBOR);
            steps.add(step);
        }
    }


    /***************************************************************************
     * Definuje pro hru doplňovací scénář, který prověří:
     * <ul>
     *   <li>Hra je schopna po ukončení opět nastartovat (restart hry).</li>
     *   <li>Hra korektně zpracuje zadání základních příkazů
     *       pro přesun, položení a zvednutí objektu
     *       v nichž nebudou zadány parametry.</li>
     *   <li>Hráč se chce přesunout do aktuálního prostoru.</li>
     *   <li>Hráč se chce přesunout do prostoru, který není sousedem
     *       aktuálního prostoru</li>
     * </ul>
     *
     * @param summary Souhrn infomrací o hře získaný z dříve testovaných scénářů
     */
    private Scenario scenarioWithAccessoryTests(ScenarioTest.Summary summary)
    {
        List<ScenarioStep> steps  = new ArrayList<ScenarioStep>();

        ScenarioStep initialStep = summary.startStep;
        String       startPlace  = initialStep.place;

        ScenarioStep finalStep = addRestartOfGame
                                (summary, startPlace, initialStep, steps);
        addArgumentLessStatement(summary, startPlace, initialStep, steps);
        addEnteringActualSpace  (summary, startPlace, initialStep, steps);
        addEnteringAFarSpace    (summary, startPlace, initialStep, steps);

        steps.add(finalStep);

        Scenario scenario = new Scenario("Kontrolní", TypeOfScenario.scGENERAL,
                       manager, steps.toArray(new ScenarioStep[steps.size()]));

        ScenarioTest test = new ScenarioTest();
        test.test(scenario);

        return scenario;
    }


    /***************************************************************************
     * Definuje pro hru doplňovací scénář, který prověří:
     * <ul>
     *   <li>Hra je schopna po ukončení opět nastartovat (restart hry).</li>
     *   <li>Hra korektně zpracuje zadání základních příkazů
     *       pro přesun, položení a zvednutí objektu
     *       v nichž nebudou zadány parametry.</li>
     *   <li>Hráč se chce přesunout do aktuálního prostoru.</li>
     *   <li>Hráč se chce přesunout do prostoru, který není sousedem
     *       aktuálního prostoru</li>
     * </ul>
     *
     * @param summary Souhrn infomrací o hře získaný z dříve testovaných scénářů
     */
    private Scenario sceanrioOfDefense_4(ScenarioTest.Summary summary)
    {
        ScenarioStep initialStep = summary.startStep;
        ScenarioStep magic = new ScenarioStep(
                             "Kouzlo " + manager.getXNAME(),
                             ScenarioStep.IGNORED_MESSAGE,
                             "Magie",
                             new String[] {summary.endStep.place},
                             new String[] {},
                             summary.endStep.bag,
                             TypeOfStep.tsNON_STANDARD);
        ScenarioStep finish = new ScenarioStep(
                            summary.basicCommands.get(TypeOfStep.tsMOVE) + " " +
                                                      summary.endStep.place,
                            ScenarioStep.IGNORED_MESSAGE,
                            summary.endStep.place,
                            summary.endStep.neighbors,
                            summary.endStep.objects,
                            summary.endStep.bag,
                            TypeOfStep.tsMOVE);
        String inititalPlace = initialStep.place;

        List<ScenarioStep>  kroky  = new ArrayList<ScenarioStep>();
        kroky.add(initialStep);
        kroky.add(magic);
        kroky.add(finish);

        Scenario scenario = new Scenario("Obhajoba_4", TypeOfScenario.scGENERAL,
                       manager, kroky.toArray(new ScenarioStep[kroky.size()]) );

        ScenarioTest test = new ScenarioTest();
        test.test(scenario);

        return scenario;
    }


    /***************************************************************************
     * Definuje pro hru doplňovací scénář, který prověří:
     * <ul>
     *   <li>Hra je schopna po ukončení opět nastartovat (restart hry).</li>
     *   <li>Hra korektně zpracuje zadání základních příkazů
     *       pro přesun, položení a zvednutí objektu
     *       v nichž nebudou zadány parametry.</li>
     *   <li>Hráč se chce přesunout do aktuálního prostoru.</li>
     *   <li>Hráč se chce přesunout do prostoru, který není sousedem
     *       aktuálního prostoru</li>
     * </ul>
     *
     * @param summary Souhrn infomrací o hře získaný z dříve testovaných scénářů
     */
    private Scenario sceanrioOfDefense_6(ScenarioTest.Summary souhrn)
    {
        final String   ZAKLÍNADLO   = "Abrakadabra";
        final String   ZKRATKA      = "Zkratka";
        final String   MAGIE        = "MAGIE";
        final String[] BATOH_M      = { MAGIE };
        final String[] NIC          = {};
        final String   START    = souhrn.startStep.place;
        final String   CÍL      = souhrn.endStep.place;
        final String   GOTO     = souhrn.basicCommands.get(TypeOfStep.tsMOVE);
        final String   GOTO_CÍL = GOTO + " " + CÍL;

        final String[] START_V0 = souhrn.startStep.neighbors;
        final int      SVL      = START_V0.length;
        final String[] SOUS_ZKR = new String[SVL + 1];
        System.arraycopy(START_V0, 0, SOUS_ZKR, 0, SVL);
        SOUS_ZKR[SVL] = ZKRATKA;


        //======================================================================
        //Použité kroky scénáře inicializované v rámci budování vektoru
        final ScenarioStep start, basic_BP, basic_S1, basic_XN, goto_ZKR,
                                  basic_S2, basic_MG, basic_GT, goto_cíl;

        //Jednotlivé kroky budovaného scénáře
        final ScenarioStep[] kroky =
        {
            start   = souhrn.startStep,

            //Zaklínadlo bez parametrů
            basic_BP = new ScenarioStep(
                      ZAKLÍNADLO,
                      ScenarioStep.IGNORED_MESSAGE,
                      start.place,
                      start.neighbors,
                      start.objects,
                      start.bag,
                      TypeOfStep.tsNON_STANDARD),

            //Zaklínadlo se špatným parametrem (pro tuto chvíli)
            basic_S1 = new ScenarioStep(
                      ZAKLÍNADLO + " " + ZAKLÍNADLO,
                      ScenarioStep.IGNORED_MESSAGE,
                      start.place,
                      start.neighbors,
                      start.objects,
                      start.bag,
                      TypeOfStep.tsNON_STANDARD),

            //Správně zadané zaklínadlo -
            //Sousedem aktuálního prostoru se stane prostor ZKRATKA,
            //který je bez sousedů a objektů
            basic_XN = new ScenarioStep(
                      "Abrakadabra " + manager.getXNAME(),
                      ScenarioStep.IGNORED_MESSAGE,
                      start.place,
                      SOUS_ZKR,
                      start.objects,
                      start.bag,
                      TypeOfStep.tsNON_STANDARD),

            //Přesun do prostoru ZKRATKA
            goto_ZKR = new ScenarioStep(
                      GOTO + " " + ZKRATKA,
                      ScenarioStep.IGNORED_MESSAGE,
                      ZKRATKA,
                      NIC,
                      NIC,
                      start.bag,
                      TypeOfStep.tsMOVE),

            //Zaklínadlo se špatným parametrem (pro tuto chvíli)
            basic_S2 = new ScenarioStep(
                      "Abrakadabra " + manager.getXNAME(),
                      ScenarioStep.IGNORED_MESSAGE,
                      ZKRATKA,
                      NIC,
                      NIC,
                      start.bag,
                      TypeOfStep.tsNON_STANDARD),

            //Správně zadané zaklínadlo -
            //Hráč vyprázdní batoh, a přesune do něj objekt MAGIE
            //Současně se cílový prostor stane sousedem prostoru ZKRATKA
            basic_GT = new ScenarioStep(
                      "Abrakadabra Abrakadabra",
                      ScenarioStep.IGNORED_MESSAGE,
                      ZKRATKA,
                      new String[] { CÍL },
                      NIC,
                      BATOH_M,
                      TypeOfStep.tsNON_STANDARD),

            //Přechod do závěrečné místnosti s magiií v batohu
            //ukončí úspěšně hru
            goto_cíl = new ScenarioStep(
                      GOTO_CÍL,
                      souhrn.endStep.message,
                      CÍL,
                      souhrn.endStep.neighbors,
                      souhrn.endStep.objects,
                      BATOH_M,
                      TypeOfStep.tsMOVE)
        };

        Scenario scenario = new Scenario("Obhajoba_6", TypeOfScenario.scGENERAL,
                                                       manager, kroky);
        ScenarioTest test = new ScenarioTest();
        test.test(scenario);

        return scenario;
    }


//== VNOŘENÉ A VNITŘNÍ TŘÍDY ===================================================
//== TESTY A METODA MAIN =======================================================
//
//    /***************************************************************************
//     * Testovací metoda.
//     */
//    public static void test()
//    {
//        GameTRunTest x = new GameTRunTest();
//    }
//    /** @param args Parametry příkazového řádku - nepoužívané. */
//    public static void main( String[] args )  {  test();  }
}
