/* Soubor je ulozen v kodovani UTF-8.
 * Kontrola kódování: Příliš žluťoučký kůň úpěl ďábelské ódy.
 */
package cz.pecinovsky.english.pojj.adv_framework.test_util;

import cz.pecinovsky.english.pojj.adv_framework.scenario.Scenario;
import cz.pecinovsky.english.pojj.adv_framework.scenario.AScenarioManager;
import cz.pecinovsky.english.pojj.adv_framework.scenario.ScenarioStep;

import cz.pecinovsky.english.pojj.adv_framework.utilities.DBG_Logger;

import java.util.Date;
import java.util.Iterator;



/*******************************************************************************
 * Instance třídy {@code ScenarioManagerTest} umějí otestovat správce scénářů.
 * Testují však pouze korektnost vlastních správců scénářů
 * a korektnost jimi spravovaných scénářů.
 * Nepokouší se pomocí scénářů zadaného správce testovat příslušnou hru.
 * To mají na starosti instance třídy {@link GameTest}.
 * <p>
 * Třída není definována jako knihovní,
 * aby mohla implementovat rozhraní {@code ITest}
 * a bylo ji tak možno použít v hromadných testech odevzdaných scénářů.
 *
 * @author    Rudolf PECINOVSKÝ
 * @version   0.00.000
 */
class ScenarioManagerTest implements ITest<AScenarioManager>
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================

    /** Logger, prostřednictvím nějž zanamenáváme veškeré informace. */
    private final static DBG_Logger DBG = DBG_Logger.getInstance();

    private static final ScenarioManagerTest singleton =
                                             new ScenarioManagerTest();



//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================
//== STATICKÝ INICIALIZAČNÍ BLOK - STATICKÝ KONSTRUKTOR ========================
//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================
//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ TŘÍDY ========================================
//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

    /***************************************************************************
     * Vypíše úvodní zprávu hry, ze které by se mělo dát odhadnout,
     * o čem hra bude.
     *
     * @param manager Správce scénářů hry, jejíž uvítání se vypisuje
     */
    public static void writeInvitation(AScenarioManager manager)
    {
        Scenario     scenario    = manager.getHappyScenario();
        ScenarioStep initialStep = scenario.iterator().next();
        DBG.info(initialStep.toString());
    }



//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    /***************************************************************************
     * Vytvoří objekt schopný testovat správce scénářů.
     */
    private ScenarioManagerTest()
    {
    }


    /***************************************************************************
     * Vrátí objekt pro testování správce scénářů.
     *
     * @return Objekt pro testování správce scénářů
     */
    public static ScenarioManagerTest getInstance()
    {
        return singleton;
    }



//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ INSTANCÍ =====================================
//== OSTATNÍ NESOUKROMÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Otestuje, že zadaný správce scénářů vyhovuje na něj kladeným požadavkům
     * a otestuje všechny jeho scénáře.
     *
     * @param manager Testovaná instance
     */
    @Override
    public void test(AScenarioManager manager)
    {
        Date startTime =  new Date();
        DBG.info("NOVY ########## START: {0, date}\n", startTime);

        writeInvitation(manager);

        verifyHappyScenario  (manager);    //Ověří, že nultým scénářem je ZÚspě
        verifyMistakeScenario(manager);    //Ověří, že prvním scénářem je ZChyb

        ScenarioTest.Summary summary = ScenarioTest.testAllScenarios(manager);
        boolean ok = summary.ok;

        Date   stopTime =  new Date();
        double duration = (stopTime.getTime() - startTime.getTime()) / 1000.0;
        DBG.info("NOVY ########## STOP: {0, date}  -  {1, number,#.#} s\n" +
                 "           Test byl {2}ÚSPĚŠNÝ\n\n" ,
                 stopTime, duration, (ok ? "" : "NE"));
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================
//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Prověří, že základní úspěšný scénář dodaný testovaným správcem
     * je prvním scénářem nabízeným jeho iterátorem.
     * 
     * @param manager Testovaný správce scénářů
     */
    private void verifyHappyScenario(AScenarioManager manager)
    {
        Scenario happyScenario  = manager.getHappyScenario();
        Scenario zerothScenario = manager.getScenario(0);

        Iterator<Scenario> iterator = manager.iterator();
        if ((happyScenario == null)          ||
            (! happyScenario.equals(zerothScenario))  ||
            (! iterator.hasNext())     ||
            (! iterator.next().equals(happyScenario)) )
        {
            throw new TestException("\nIterátor správce scénářů musí vždy "  +
                             "nabídnout jako první základní úspěšný scénář," +
              "\nkterý musí být současně získatelný jako scénář s indexem 0");
        }
    }


    /***************************************************************************
     * Prověří, že základní chybový scénář dodaný testovaným správcem
     * je druhým scénářem nabízeným jeho iterátorem.
     * 
     * @param manager Testovaný správce scénářů
     */
    private void verifyMistakeScenario(AScenarioManager manager)
    {
        Scenario mistakeScenario = manager.getMistakeScenario();
        Scenario firstScenario   = manager.getScenario(1);

        Iterator<Scenario> iterator = manager.iterator();
        if ((mistakeScenario != null)        &&
            (mistakeScenario.equals(firstScenario))  &&
            (iterator.hasNext()))
        {
            iterator.next();
            if (iterator.hasNext())  {
                Scenario druhý = iterator.next();
                if (druhý.equals(mistakeScenario)) {
                    return;                     //==========>
                }
            }
        }
        throw new TestException("\nIterátor správce scénářů musí vždy " +
                             "nabídnout jako druhý základní chybový scénář," +
              "\nkterý musí být současně získatelný jako scénář s indexem 1");
    }



//== INTERNÍ DATOVÉ TYPY =======================================================
//== TESTOVACÍ METODY A TŘÍDY ==================================================
//
//    /***************************************************************************
//     * Testovací metoda.
//     */
//    public static void test()
//    {
//        ScenarioManagerTest inst = new ScenarioManagerTest();
//    }
//    /** @param args Parametry příkazového řádku - nepoužívané. */
//    public static void main( String[] args )  {  test();  }
}
