package cz.pecinovsky.english.pojj.adv_framework.test_util.default_game.hra;

import cz.pecinovsky.english.pojj.adv_framework.game_gui.IListener;
import  cz.pecinovsky.english.pojj.adv_framework.game_txt.ICommand;

import  cz.pecinovsky.english.pojj.adv_framework.game_txt.INamed;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;


/*******************************************************************************
 * Instance třídy {@code APříkaz} (přesněji instance jejích potomků) mají
 * na starost interpretaci přikazů zadávaných uživatelem hrajícím danou hru.
 * Třída {@code APříkaz} je rodičovskou třídou všech přikazů.
 *
 * @author    Rudolf PECINOVSKÝ
 * @version   5.00
 */
abstract class APříkaz implements ICommand
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================

    private static final String UVÍTÁNÍ = Texty.zCELÉ_UVÍTÁNÍ;
//            "\nVítáme vás ve vašem novém bytě. Jistě máte hlad." +
//            "\nNajděte v bytě ledničku - tam vás čeká svačina." +
//            "\n\nNacházíte se v místnosti: Předsíň." +
//            "\nV místnosti se nachází: Botník, Deštník" +
//            "\nMůžete se přesunout do místností: Ložnice, Obývák, Koupelna";

    private static final String PRÁZDNÝ = Texty.zPRÁZDNÝ_PŘÍKAZ;
//            "Zadal(a) jste prázdný příkaz. Chcete-li poradit, zadejte příkaz ?";

    private static final String NEZNÁM = Texty.zNEZNÁMÝ_PŘÍKAZ;
//            "Tento příkaz neznám. Chcete-li poradit, zadejte příkaz ?";


    private static final Map<String, ICommand> název2příkaz =
                                             new HashMap<String, ICommand>();

    /** Teoreticky by následující příkazy stačilo vytvořit např. ve statickém
     *  inicializačním bloku. Tady je každému příkazu přiřazena konstanta
     *  pouze proto, že se mi zdálo, že by se mi to mohlo při ladění hodit. */
    private static final APříkaz
            PŘÍKAZ_HELP  = new PříkazHelp(),
            PŘÍKAZ_JDI   = new PříkazJdi(),
            PŘÍKAZ_OTEVRI= new PříkazOtevři(),
            PŘÍKAZ_NASAD = new PříkazNasaď(),
            PŘÍKAZ_PODLOZ= new PříkazPodlož(),
            PŘÍKAZ_POLOZ = new PříkazPolož(),
            PŘÍKAZ_PRECTI= new PříkazPřečti(),
            PŘÍKAZ_VEZMI = new PříkazVezmi(),
            PŘÍKAZ_ZAVRI = new PříkazZavři(),
            PŘÍKAZ_KONEC = new PříkazKonec();



//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================

    /** Příznak toho, je-li hra rozehraná nebo
     *  je-li skončená a bude se startovat nová. */
    private static boolean nehrajeme = true;



//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

    private final String název;
    private final String popis;



//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ TŘÍDY ========================================

    /***************************************************************************
     * Vrátí kolekci všech příkazů použitelných ve hře..
     *
     * @return Požadovaná kolekce
     */
    static Collection<ICommand> getPříkazy()
    {
        return Collections.unmodifiableCollection(název2příkaz.values());
    }



//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

    /***************************************************************************
     * Zpracuje zadany prikaz a vrati text zpravy pro uzivatele.
     *
     * @param příkaz Zadavany prikaz
     * @return Textova odpoved hry na zadany prikaz
     */
    static String zpracujPříkaz(String příkaz)
    {
        příkaz = příkaz.toLowerCase().trim();
        if (nehrajeme) {
            return start(příkaz);
        }
        if (příkaz.equals("")) {
            return PRÁZDNÝ;
        }
        if (Stav.ROZHOVOR) {
            return Rozhovor.reaguj(příkaz);                 //==========>
        }
        String[] slova = příkaz.split("\\s+");
        ICommand  povel = název2příkaz.get(slova[0]);
        if (povel == null) {
            return NEZNÁM;                                  //==========>
        }
//        if (DBG.DEBUG < 0) {
//            povel.getParametry();
//        }

        String[] parametry = new String[slova.length-1];
        System.arraycopy(slova, 1, parametry, 0, parametry.length);

        String  odpověď = povel.execute(parametry);
        odpověď = doplň(odpověď);
        return  odpověď;
    }


    /***************************************************************************
     * Oznámení o ukončení hry.
     * Příští volání metody {@link #zpracujPříkaz(String)}
     * bude opět prvním voláním v nové hře.
     */
    public static void konec() {
        nehrajeme = true;
    }



//+++ Přidáno pro rozšířené zadnání v předmětu 4IT115 ++++++++++++++++++++++++++

    /***************************************************************************
     * Přidá zadaného posluchače do seznamu posluchačů,
     * kterým hra oznamuje začátek a konec rozhovoru.
     *
     * @param posluchač Přihlašovaný posluchač
     */
    public static void přihlašPosluchačeRozhovoru(IListener<Boolean> posluchač)
    {
        Rozhovor.přihlašPosluchačeRozhovoru(posluchač);
    }


    /***************************************************************************
     * Odebere zadaného posluchače ze seznamu posluchačů,
     * kterým hra oznamuje začátek a konec rozhovoru.
     *
     * @param posluchač Odhlašovaný posluchač
     */
    public static void odhlašPosluchačeRozhovoru(IListener<Boolean> posluchač)
    {
        Rozhovor.odhlašPosluchačeRozhovoru(posluchač);
    }



//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    /***************************************************************************
     * Vytvoří nový příkaz se zadaným názvem a popisem své funkce.
     *
     * @param název Název vytvářeného příkazu
     * @param popis Stručný popis funkce, použití a účelu příkazu
     */
    APříkaz (String název, String popis)
    {
        this.název = název;
        this.popis = popis;
        název2příkaz.put(název.toLowerCase(), this);
    }


//== ABSTRAKTNÍ METODY =========================================================

    /***************************************************************************
     * Metoda realizující reakci hry na zadaní daného příkazu.
     * Počet parametru je závislý na konkrétním příkazu, např. příkazy Konec
     * a Nápověda nemají parametry, příkazy Jdi a Vezmi mají jeden parametr
     * příkaz Podlož muže mít dva parametry atd.
     *
     * @param parametry Parametry příkazu;
     *                  jejich počet muže byt pro každý příkaz jiný
     * @return Text zprávy vypsané po provedení příkazu
     */
    @Override
    public abstract String execute( String... parametry );



//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ INSTANCÍ =====================================

    /***************************************************************************
     * Vraci nazev prikazu, tj. text, ktery musi hrac zadat
     * pro vyvolani daneho prikazu
     *
     * @return Nazev prikazu
     */
    @Override
    public String getName()
    {
        return název;
    }


    /***************************************************************************
     * Vraci popis prikazu s vysvetlenim jeho funkce
     * a vyznamu jednotlivych parametru.
     *
     * @return Popis prikazu
     */
    @Override
    public String getDescription()
    {
        return popis;
    }


    /***************************************************************************
     * Vrátí informaci o tom, jeli hra již skočena a lze-li proto začít novou.
     * @return
     */
    static boolean isSkončeno()
    {
        return nehrajeme;
    }



//+++ Přidáno pro rozšířené zadnání v předmětu 4IT115 ++++++++++++++++++++++++++
//
//    /***************************************************************************
//     * Metoda vrátí pole parametrů daného příkazu. Každý parametr je přitom
//     * schopen vrátit množinu hodnot, které v dané situaci přicházejí
//     * pro daný příkaz na pozici příslušného parametru v úvahu.
//     *
//     * @return Požadované pole parametrů
//     * @throws UnsupportedOperationException v případě, kdy příkaz tuto
//     *                                       funkci nepodporuje
//     */
//    @Override
//    public IParametr[] getParametry()
//    {
//        throw new UnsupportedOperationException();
//    }
//
//
//
//== OSTATNÍ NESOUKROMÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Vrátí název dané instance.
     *
     * @return Nazev instance
     */
    @Override
    public String toString()
    {
        return název;
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================

    /***************************************************************************
     * Vrátí text s informacemi o aktuálním stavu hry.
     * 603 802 220
     * @return Požadovaný informační text
     */
    private static String doplň(String  odpověď)
    {
        StringBuilder sb = new StringBuilder(odpověď);
        Collection<INamed> kolekce;

        Místnost am = Místnost.getAktuálníMístnost();
        připoj(sb, "Sousedé:  ", am.getNeighbors());
        připoj(sb, "Předměty: ", am.getObjects());
        připoj(sb, "Batoh:    ", Batoh.getBatoh().getObjects());

        return sb.toString();
    }


    /***************************************************************************
     * Do zadaného stringbuilderu přidá zadaný titulek
     * následovaný vyjmenovanými položkami ze zadané kolekce.
     *
     * @param sb       Doplňovaný StringBuilder
     * @param titulek  Přidávaný titulek
     * @param objekty  Kolekce přidávaných objektů
     */
    private static void připoj(StringBuilder sb, String titulek,
                        Collection<? extends INamed> objekty)
    {
        sb.append('\n').append(titulek).append(objekty);
    }


    /***************************************************************************
     * Odstartuje novou hru.
     */
    private static String start(String příkaz)
    {
        if (příkaz.equals("")) {
            Místnost.inicializuj();
            Batoh   .inicializuj();
            nehrajeme = false;
            return UVÍTÁNÍ;                             //==========>
        } else {
            throw new IllegalStateException(
                                "Prvním příkazem není bezejmenný příkaz.");
        }
    }



//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================
//== VNOŘENÉ A VNITŘNÍ TŘÍDY ===================================================
//== TESTY A METODA MAIN =======================================================
//
//    /***************************************************************************
//     * Testovací metoda.
//     */
//    public static void test()
//    {
//        APříkaz x = new APříkaz();
//    }
//    /** @param args Parametry příkazového řádku - nepoužívané. */
//    public static void main( String[] args )  {  test();  }
}
