/* Soubor je ulozen v kodovani UTF-8.
 * Kontrola kódování: Příliš žluťoučký kůň úpěl ďábelské ódy.
 */
package cz.pecinovsky.english.pojj.adv_framework.test_util.default_game.hra;

import cz.pecinovsky.english.pojj.adv_framework.game_gui.IBagG;
import cz.pecinovsky.english.pojj.adv_framework.game_gui.IListener;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;



/*******************************************************************************
 * Instance třídy {@code Batoh} představují úložiště, do nichž si hráči
 * odkládají sebrané předměty či jejch ekvivalenty.
 * Instance třídy je definovaná jako jedináček,
 * protože celá hra vystačí s jediným batohem.
 *
 * @author    Rudolf PECINOVSKY
 * @version   5.0
 */
public class Batoh implements IBagG
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================

    /** Maximální kapacita batohu - v této hře dvě ruce. */
    private static final int KAPACITA = 2;


    /** Schránka na přenášené předměty. V případě této hry se ale předměty
     *  přenášejí v rukou, a proto může hráč níst současně nejvýše dva. */
    private static final Batoh batoh = new Batoh();



//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================
//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

    /** Kolekce předmětů v batohu. */
    private final Collection<Předmět> obsah = new ArrayList<Předmět>();

    /** Posluchači očekávající oznámení o změně předmětů v batohu. */
    private final Collection<IListener<IBagG>> posluchači =
                                       new ArrayList<IListener<IBagG>>();



//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================
//== PŘÍSTUPOVÉ METODY ATRIBUTŮ TŘÍDY ==========================================
//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    static Batoh getBatoh()
    {
        return batoh;
    }



    /***************************************************************************
     * Vytvoří prázdný batoh.
     * Konstruktor je soukromý, aby nebylo možno omylem vytvářet další batohy.
     */
    private Batoh()
    {
        //Pro vytvoření batohu stačí prázdný konstruktor -
        //vše potřebné je vykonáno v deklaracích atributů
    }



//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY INSTANCÍ ================================================

    /***************************************************************************
     * Vrátí kapacitu batohu, tj. kolik předmětů se do něj vejde.
     *
     * @return Kapacita batohu
     */
    @Override
    public int getCapacity()
    {
        return 2;
    }


    /***************************************************************************
     * Vrátí kolekci předmětů uložených v batohu.
     *
     * @return Kolekce předmětů v batohu
     */
    @Override
    public Collection<Předmět> getObjects()
    {
        return Collections.unmodifiableCollection(obsah);
    }



//== OSTATNÍ NESOUKROMÉ  METODY INSTANCÍ =======================================

    /***************************************************************************
     * Inicializuje jedináčka a připraví jej pro novou hru.
     */
    static void inicializuj()
    {
        batoh.obsah.clear();
    }


    /***************************************************************************
     * Zjistí, zda batoho obsahuje předmět zadaný svým názvem.
     *
     * @param  název Název předmětu v batohu
     * @return {@code true} pokud je předmět v batohu, jinak {@code falše}
     */
    boolean obsahuje( String název ) {
        Předmět našel = (Předmět)Utility.najdi(název, obsah);
        return (našel != null);
    }


    /***************************************************************************
     * Přidá zadaný předmět do batohu.
     *
     * @param  předmět Předmět přidávaný do batohu
     * @return {@code true} pokud byl předmět do batohu vložen
     */

    boolean přidej(Předmět předmět) {
        boolean zvedám = (obsah.size() < KAPACITA)  &&  obsah.add( předmět );
        if (zvedám) {
            obvolejPosluchače();
        }
        return zvedám;
    }


    /***************************************************************************
     * Vyjme z batohu předmět zadaný svým názvem.
     *
     * @param  název Název předmětu vybíraného z batohu
     * @return Byl-li předmět v batohu, vrátí odkaz na předmět,
     *         jinak vrátí prázdný odkaz {@code null}
     */
    Předmět vyjmi(String název) {
        Předmět našel = (Předmět)Utility.najdi(název, obsah);
        if (našel != null) {
            obsah.remove(našel);
            obvolejPosluchače();
        }
        return našel;
    }


    /** {@inheritDoc} */
    @Override
    public String toString()  {
        return "Batoh obsahující " + obsah;
    }



//+++ Přidáno pro rozšířené zadnání v předmětu 4IT115 ++++++++++++++++++++++++++

    /***************************************************************************
     * Přidá zadaného posluchače do seznamu posluchačů,
     * které zpravuje o výskytu očekávané události
     *
     * @param posluchač Přihlašovaný posluchač
     */
    public void přihlašPosluchače(IListener<IBagG> posluchač)
    {
        posluchači.add(posluchač);
    }


    /***************************************************************************
     * Odebere zadaného posluchače ze seznamu posluchačů,
     * které zpravuje o výskytu očekávané události
     *
     * @param posluchač Odhlašovaný posluchač
     */
    public void odhlašPosluchače(IListener<IBagG> posluchač)
    {
        posluchači.remove(posluchač);
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================
//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Obvolá všechny přihlášené posluchače, čímž jím oznámí,
     * že se obsah batohu změnil.
     */
    private void obvolejPosluchače()
    {
        for (IListener<IBagG> posluchač : posluchači) {
            posluchač.notice(this);
        }
    }



//== VNOŘENÉ A VNITŘNÍ TŘÍDY ===================================================
//== TESTOVACÍ METODY A TŘÍDY ==================================================
//
//     /***************************************************************************
//      * Testovací metoda.
//      */
//     public static void test()
//     {
//          Batoh instance = new Batoh();
//     }
//     /** @param args Parametry příkazového řádku - nepoužité. */
//     public static void main( String... args ) { test(); }
}
