package cz.pecinovsky.english.pojj.adv_framework.test_util.default_game.hra;

import cz.pecinovsky.english.pojj.adv_framework.test_util.default_game.
       data.DataPkg;

import cz.pecinovsky.english.pojj.adv_framework.game_gui.IPlaceG;
import cz.pecinovsky.english.pojj.adv_framework.game_gui.Commands;
import cz.pecinovsky.english.pojj.adv_framework.game_gui.IBagG;
import cz.pecinovsky.english.pojj.adv_framework.game_gui.IGameG;
import cz.pecinovsky.english.pojj.adv_framework.game_gui.IListener;

import cz.pecinovsky.english.pojj.adv_framework.game_txt.ICommand;

import cz.pecinovsky.english.pojj.adv_framework.scenario.AScenarioManager;

import java.net.URL;

import java.util.Collection;

import javax.swing.ImageIcon;


import static cz.pecinovsky.english.pojj.adv_framework.test_util.default_game.
              hra.Texty.*;



/*******************************************************************************
 * Instance třídy {@code DefaultGame} představují hru, při níž se hráč ocitne
 * v malém bytě, který je třeba projít a najít v něm ledničku.
 * Mají na starosti logiku hry.
 * Jsou schopny akceptovat jednotlivé příkazy a poskytovat informace
 * o průběžném stavu hry a jejích součástech.
 *
 * @author    Rudolf PECINOVSKÝ
 * @version   5.0
 */
public class DefaultGame implements IGameG
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================

    /** Zda bude používán spráce scénářů s konstantami nebo literály. */
    private static final boolean SPRÁVCE_S_KONSTANTAMI = false;

    /** Aktuálně provozovaná hra. Je-li tento odkaz prázdný, není aktuálně
     *  provozována žádná hra a může se spustit nová. */
    private static DefaultGame aktuálníHra = new DefaultGame();

    /** Přepravka s názvy čtyř povinných příkazů. */
    private static final Commands příkazy =
                     new Commands(pJDI, pPOLOŽ, pVEZMI, pKONEC);



//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================
//== STATICKÝ INICIALIZAČNÍ BLOK - STATICKÝ KONSTRUKTOR ========================
//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

    /** Správce scénářů, s jejichž pomocí je možno hru testovat
     * nebo domonstrovat její možný průběh. */
    private final AScenarioManager správceScénářů =
                            (SPRÁVCE_S_KONSTANTAMI  ?  new SprávceKonstanta()
                                                    :  new SprávceLiterál  ());



//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================
//== PŘÍSTUPOVÉ METODY ATRIBUTŮ TŘÍDY ==========================================

    /***************************************************************************
     * Vrátí odkaz na aktuálně provozovanou instanci hry.
     * Metoda slouží k tomu, aby mohli někžteré metody získat odkaz
     * na aktuální hru a od ní pak získat jinak nedostupné informace.
     *
     * @return Požadovaný odkaz
     */
    public static DefaultGame getInstance()
    {
        return aktuálníHra;
    }



//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    /***************************************************************************
     * Vytvoří novou hru a nastaví jí správce scénářů.
     */
    private DefaultGame()
    {
        DefaultGame já = this;
        správceScénářů.setGame(já);
    }



//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY INSTANCÍ ================================================

    /***************************************************************************
     * Vrátí jméno autora scénáře ve formátu "PŘÍJMENÍ Křestní",
     * tj. neprve příjmení psané velkými písmeny a za ním křestní jméno,
     * u nějž bude velké pouze první písmeno a ostatní písmena budou malá.
     *
     * @return Jméno autora/autorky programu ve tvaru PŘÍJMENÍ Křestní
     */
    @Override
    public String getAuthor()
    {
        return správceScénářů.getAuthor();
    }


    /***************************************************************************
     * Vrátí x-name autora/autorky programu zapsané velkými písmeny.<p>
     *
     * @return X-name autora/autorky programu
     */
    @Override
    public String getXNAME()
    {
        return správceScénářů.getXNAME();
    }


    /***************************************************************************
     * Vrátí odkaz na aktuální místnost,
     * tj. na místnost, v niž se pravé nachází hráč.
     *
     * @return Požadovaný odkaz
     */
    @Override
    public Místnost getActualPlace()
    {
        return Místnost.getAktuálníMístnost();
    }


    /***************************************************************************
     * Vrátí odkaz na batoh, do nějž bude hráč ukládat sebrané předměty.
     *
     * @return Požadovaný odkaz
     */
    @Override
    public Batoh getBag()
    {
        return Batoh.getBatoh();
    }


    /***************************************************************************
     * Vrátí kolekci odkazů na jednotlivé místnosti.
     *
     * @return Požadovaná kolekce
     */
    @Override
    public Collection<Místnost> getAllPlaces()
    {
        return Místnost.getMístnosti();
    }


    /***************************************************************************
     * Vrátí kolekci všech příkazů použitelných ve hře.
     *
     * @return Požadovaná kolekce
     */
    @Override
    public Collection<ICommand> getAllCommands()
    {
        return APříkaz.getPříkazy();
    }


    /***************************************************************************
     * Vrátí informaci o tom, je-li hra již ukončena.
     *
     * @return Požadovaná informace
     */
    @Override
    public boolean isReady()
    {
        return APříkaz.isSkončeno();
    }


    /***************************************************************************
     * Vrátí správce scénářů, které umožní prověřit funkčnost hry
     * a/nebo demonstrovat její průběh.
     *
     * @return Správce scénářů dané hry
     */
    @Override
    public AScenarioManager getScenarioManager()
    {
        return správceScénářů;
    }



//+++ Přidáno pro rozšířené zadnání v předmětu 4IT115 ++++++++++++++++++++++++++

    /***************************************************************************
     * Vrátí vstupní znakový proud obsahující stránku s nápovědou.
     *
     * @return Požadovaný odkaz
     */
    @Override
    public URL getHelpURL()
    {
        Class<DataPkg> classDataPkg = DataPkg.class;
        URL            url          = classDataPkg.getResource("Help.htm");
//        URL          url          = DataPkg.class.getResource("Help.htm");
        return url;
    }


    /***************************************************************************
     * Vrátí obrázek s plánkem prostoru, v němž se hraje.
     *
     * @return Požadovaný odkaz
     */
    @Override
    public ImageIcon getMap()
    {
        URL url = DataPkg.class.getResource("Plánek.png");
        ImageIcon obr   = new ImageIcon(url);
        return obr;
    }


    /***************************************************************************
     * Vrátí odkaz na přepravku s názvy příkazů pro přesun do jiné místnosti,
     * zvednutí předmětu a jeho položení.
     *
     * @return Požadovaný odkaz
     */
    @Override
    public Commands getBasicCommands()
    {
        return příkazy;
    }



//== OSTATNÍ NESOUKROMÉ  METODY INSTANCÍ =======================================

    /***************************************************************************
     * Ukončí celou hru a vrátí alokované prostředky.
     */
    @Override
    public void stop()
    {
        APříkaz.konec();
    }


    /***************************************************************************
     * Zpracuje zadaný příkaz a vrátí text zprávy pro uživatele.
     *
     * @param prikaz Zadávaný příkaz
     * @return Textová odpověď hry na zadaný příkaz
     */
    @Override
    public String executeCommand( String prikaz )
    {
        return APříkaz.zpracujPříkaz(prikaz);
    }


    /***************************************************************************
     * Přidá zadaného posluchače do seznamu posluchačů,
     * které zpravuje o změně množiny předmětů v batohu.
     *
     * @param listener Přihlašovaný posluchač
     */
    @Override
    public void addBagListener(IListener<IBagG> listener)
    {
        Batoh.getBatoh().přihlašPosluchače(listener);
    }


    /***************************************************************************
     * Odebere zadaného posluchače ze seznamu posluchačů,
     * které zpravuje o změně množiny předmětů v batohu.
     *
     * @param listener Odhlašovaný posluchač
     */
    @Override
    public void removeBagListener(IListener<IBagG> listener)
    {
        Batoh.getBatoh().odhlašPosluchače(listener);
    }


    /***************************************************************************
     * Přidá zadaného posluchače do seznamu posluchačů,
     * které zpravuje o změně množiny předmětů v aktuální místnosti.
     *
     * @param listener Přihlašovaný posluchač
     */
    @Override
    public void addObjectListener(IListener<IPlaceG> listener)
    {
        Místnost.přihlašPosluchačePředmětů(listener);
    }


    /***************************************************************************
     * Odebere zadaného posluchače ze seznamu posluchačů,
     * které zpravuje o změně množiny předmětů v aktuální místnosti.
     *
     * @param listener Odhlašovaný posluchač
     */
    @Override
    public void removeObjectListener(IListener<IPlaceG> listener)
    {
        Místnost.odhlašPosluchačePředmětů(listener);
    }


    /***************************************************************************
     * Přidá zadaného posluchače do seznamu posluchačů,
     * které zpravuje o změně množiny sousedů aktuální místnosti.
     *
     * @param listener Přihlašovaný posluchač
     */
    @Override
    public void addNeighborsListener(IListener<IPlaceG> listener)
    {
        Místnost.přihlašPosluchačeSousedů(listener);
    }


    /***************************************************************************
     * Odebere zadaného posluchače ze seznamu posluchačů,
     * které zpravuje o změně množiny sousedů aktuální místnosti.
     *
     * @param listener Odhlašovaný posluchač
     */
    @Override
    public void removeNeighborsListener(IListener<IPlaceG> listener)
    {
        Místnost.odhlašPosluchačeSousedů(listener);
    }


    /***************************************************************************
     * Přidá zadaného posluchače do seznamu posluchačů,
     * kterým hra oznamuje začátek a konec rozhovoru.
     *
     * @param listener Přihlašovaný posluchač
     */
    @Override
    public void addDialogListener(IListener<Boolean> listener)
    {
        APříkaz.přihlašPosluchačeRozhovoru(listener);
    }


    /***************************************************************************
     * Odebere zadaného posluchače ze seznamu posluchačů,
     * kterým hra oznamuje začátek a konec rozhovoru.
     *
     * @param listener Odhlašovaný posluchač
     */
    @Override
    public void removeDialogListener(IListener<Boolean> listener)
    {
        APříkaz.odhlašPosluchačeRozhovoru(listener);
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================
//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================
//== VNOŘENÉ A VNITŘNÍ TŘÍDY ===================================================
//== TESTOVACÍ METODY A TŘÍDY ==================================================
//
//     /***************************************************************************
//      * Testovací metoda.
//      */
//     public static void test()
//     {
//        DefaultGame hra = getInstance();
//        _Test_101 test = _Test_101.getInstance(hra);
//        test.testGame();
//     }
//     /** @param args Parametry příkazového řádku - nepoužité. */
//     public static void main( String... args ) { test(); }
}
