package cz.pecinovsky.english.pojj.adv_framework.test_util.default_game.hra;

import cz.pecinovsky.english.pojj.adv_framework.test_util.default_game.
       data.DataPkg;

import cz.pecinovsky.english.pojj.adv_framework.game_gui.IObjectG;

import java.net.URL;

import javax.swing.Icon;
import javax.swing.ImageIcon;



/*******************************************************************************
 * Instance třídy {@code Předmět} představuji předměty v místnostech.
 * Některé z nich jsou přenositelné, jiné ne.
 *
 * @author    Rudolf PECINOVSKY
 * @version   5.0
 */
public class Předmět implements IObjectG
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================

    /** Prefix určující, že předmět nepůjde zvednout. */
    public static final char TĚŽKÝ = '#';

    /** Prefix určující, že se jkedná o alkoholikcý nápoj. */
    public static final char ALKOHOL = '@';

    /** Univerzální předmět zastupující všechny nezvednutelné předměty. */
    public static final Předmět TÍHA = new Předmět( TĚŽKÝ + "TÍHA" );



//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================
//== STATICKÝ INICIALIZAČNÍ BLOK - STATICKÝ KONSTRUKTOR ========================
//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

    private final boolean   zvednutelný;
    private final String    název;
    private final boolean   alkoholický;
    private final ImageIcon obrázek;


//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================
//== PŘÍSTUPOVÉ METODY ATRIBUTŮ TŘÍDY ==========================================
//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    /***************************************************************************
     * Vytvoří nový předmět se zadaným názvem, zvednutelností
     * a informací o tom, zda se jedná a alkoholický nápoj.
     * Je-li prvním znakem názvu přemětu znak # označující nevzednutelné
     * předměty, bude z názvu odebrán a předmětu bude označen za nezvednutelný.
     * Je-li prvním znakem názvu přemětu znak @ označující alkoholické nápoje,
     * bude z názvu odebrán a předmětu bude označen za alkoholický nápoj.
     *
     * @param název Název předmětu <br>
     *              - začíná-li znakem #, není předmět zvednutelný,<br>
     *              - začíná-li znakem @, jedná se o alkoholickýá nápoj
     */
    public Předmět(String název)
    {
        boolean a = false,
                z = true;

        switch (název.charAt(0))
        {
            case TĚŽKÝ:
                název = název.substring(1);
                z = false;
                break;

            case ALKOHOL:
                název = název.substring(1);
                a = true;
                break;
        }
        this.název       = název;
        this.alkoholický = a;
        this.zvednutelný = z;

        String výsledek;
        URL url = DataPkg.class.getResource(název+".png");
        if (url == null) {
            obrázek = null;
            výsledek = "_NENÍ_";
        }
        else {
            this.obrázek = new ImageIcon(url);
            výsledek = "Načten";
        }
        System.out.println("   " + výsledek + " obrázek předmětu: " + název);
    }



//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY INSTANCÍ ================================================

    /***************************************************************************
     * Vrátí název dané instance.
     *
     * @return Nazev instance
     */
    @Override
    public String getName()
    {
        return název;
    }


    /***************************************************************************
     * Vrátí informaci o tom, dá-li se předmět zvednout a vložit do batohu.
     *
     * @return {@code true} - dá se zvednout, jinak {@code false}
     */
    @Override
    public boolean isTransportable()
    {
        return zvednutelný;
    }


    /***************************************************************************
     * Vrátí informaci o tom, je-li daný předmět alkoholickým nápojem.
     *
     * @return {@code true} jedná-li se o alkoholický nápoj, jinak {@code false}
     */
    public boolean isAlkoholicky() {
        return alkoholický;
    }


    /***************************************************************************
     * Vrátí odkaz na obrázek daného předmětu.
     *
     * @return  Požadovaný odkaz
     */
    @Override
    public Icon getPicture()
    {
        return obrázek;
    }



//== OSTATNÍ NESOUKROMÉ  METODY INSTANCÍ =======================================

    /***************************************************************************
     * Vrátí název dané instance.
     *
     * @return Nazev instance
     */
    @Override
    public String toString()
    {
        return název;
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================
//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================
//== VNOŘENÉ A VNITŘNÍ TŘÍDY ===================================================
//== TESTOVACÍ METODY A TŘÍDY ==================================================
//
//     /***************************************************************************
//      * Testovací metoda.
//      */
//     public static void test()
//     {
//          Předmět instance = new Předmět();
//     }
//     /** @param args Parametry příkazového řádku - nepoužité. */
//     public static void main( String... args ) { test(); }
}
